@@ -58,6 +58,9 @@
 #define COMPUTER_NAME_KEY       "/apps/nautilus/desktop/computer_icon_name"
 #define MAX_ITEMS_OR_SUBMENU    5
 
+G_DEFINE_TYPE (PanelPlaceMenuItem, panel_place_menu_item, GTK_TYPE_IMAGE_MENU_ITEM);
+G_DEFINE_TYPE (PanelDesktopMenuItem, panel_desktop_menu_item, GTK_TYPE_IMAGE_MENU_ITEM);
+
 #define PANEL_PLACE_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_PLACE_MENU_ITEM, PanelPlaceMenuItemPrivate))
 #define PANEL_DESKTOP_MENU_ITEM_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_DESKTOP_MENU_ITEM, PanelDesktopMenuItemPrivate))
 
@@ -83,9 +86,6 @@ struct _PanelDesktopMenuItemPrivate {
 	guint        append_lock_logout : 1;
 };
 
-static GObjectClass *place_parent_class;
-static GObjectClass *desktop_parent_class;
-
 static GnomeVFSVolumeMonitor *volume_monitor = NULL;
 
 static void
@@ -692,22 +692,49 @@ panel_place_menu_item_volume_changed (GnomeVFSVolumeMonitor *monitor,
 	panel_place_menu_item_recreate_menu (place_menu);
 }
 
+static void
+panel_desktop_menu_item_append_menu (GtkWidget *menu,
+				     gpointer   data)
+{
+	PanelDesktopMenuItem *parent;
+	gboolean              add_separator;
+	GList                *children;
+	GList                *last;
+
+	parent = PANEL_DESKTOP_MENU_ITEM (data);
+
+	add_separator = FALSE;
+	children = gtk_container_get_children (GTK_CONTAINER (menu));
+	last = g_list_last (children);
+
+	if (last != NULL)
+		add_separator = !GTK_IS_SEPARATOR (GTK_WIDGET (last->data));
+
+	g_list_free (children);
+
+	if (add_separator)
+		add_menu_separator (menu);
+
+	panel_menu_items_append_from_desktop (menu, "yelp.desktop", NULL);
+	panel_menu_items_append_from_desktop (menu, "gnome-about.desktop", NULL);
+
+	if (parent->priv->append_lock_logout)
+		panel_menu_items_append_lock_logout (menu);
+}
+
 static GtkWidget *
 panel_desktop_menu_item_create_menu (PanelDesktopMenuItem *desktop_item)
 {
 	GtkWidget *desktop_menu;
 
-	desktop_menu = panel_create_menu ();
+	desktop_menu = create_applications_menu ("settings.menu", NULL);
 
-	panel_menu_items_append_from_desktop (desktop_menu, "gnomecc.desktop", NULL);
-
-	add_menu_separator (desktop_menu);
-
-	panel_menu_items_append_from_desktop (desktop_menu, "yelp.desktop", NULL);
-	panel_menu_items_append_from_desktop (desktop_menu, "gnome-about.desktop", NULL);
-
-	if (desktop_item->priv->append_lock_logout)
-		panel_menu_items_append_lock_logout (desktop_menu);
+	g_object_set_data (G_OBJECT (desktop_menu),
+			   "panel-menu-append-callback",
+			   panel_desktop_menu_item_append_menu);
+	g_object_set_data (G_OBJECT (desktop_menu),
+			   "panel-menu-append-callback-data",
+			   desktop_item);
 
 	return desktop_menu;
 }
@@ -752,7 +779,7 @@ panel_place_menu_item_finalize (GObject *object)
 					     menuitem->priv->volume_unmounted_id);
 	menuitem->priv->volume_unmounted_id = 0;
 
-	place_parent_class->finalize (object);
+	G_OBJECT_CLASS (panel_place_menu_item_parent_class)->finalize (object);
 }
 
 static void
@@ -763,12 +790,11 @@ panel_desktop_menu_item_finalize (GObject *object)
 	if (menuitem->priv->append_lock_logout)
 		panel_lockdown_notify_remove (G_CALLBACK (panel_desktop_menu_item_recreate_menu),
 					      menuitem);
-	desktop_parent_class->finalize (object);
+	G_OBJECT_CLASS (panel_desktop_menu_item_parent_class)->finalize (object);
 }
 
 static void
-panel_place_menu_item_instance_init (PanelPlaceMenuItem      *menuitem,
-				     PanelPlaceMenuItemClass *klass)
+panel_place_menu_item_init (PanelPlaceMenuItem *menuitem)
 {
 	char *bookmarks_filename;
 	char *bookmarks_uri;
@@ -838,8 +864,7 @@ panel_place_menu_item_instance_init (PanelPlaceMenuItem      *menuitem,
 }
 
 static void
-panel_desktop_menu_item_instance_init (PanelDesktopMenuItem      *menuitem,
-				       PanelDesktopMenuItemClass *klass)
+panel_desktop_menu_item_init (PanelDesktopMenuItem *menuitem)
 {
 	menuitem->priv = PANEL_DESKTOP_MENU_ITEM_GET_PRIVATE (menuitem);
 }
@@ -849,7 +874,6 @@ panel_place_menu_item_class_init (PanelPlaceMenuItemClass *klass)
 {
 	GObjectClass *gobject_class = (GObjectClass   *) klass;
 
-	place_parent_class = g_type_class_peek_parent (klass);
 	gobject_class->finalize  = panel_place_menu_item_finalize;
 
 	g_type_class_add_private (klass, sizeof (PanelPlaceMenuItemPrivate));
@@ -860,91 +884,32 @@ panel_desktop_menu_item_class_init (PanelDesktopMenuItemClass *klass)
 {
 	GObjectClass *gobject_class = (GObjectClass   *) klass;
 
-	desktop_parent_class = g_type_class_peek_parent (klass);
 	gobject_class->finalize  = panel_desktop_menu_item_finalize;
 
 	g_type_class_add_private (klass, sizeof (PanelDesktopMenuItemPrivate));
 }
 
-GType
-panel_place_menu_item_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (PanelPlaceMenuItemClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) panel_place_menu_item_class_init,
-			NULL,
-			NULL,
-			sizeof (PanelPlaceMenuItem),
-			0,
-			(GInstanceInitFunc) panel_place_menu_item_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (GTK_TYPE_IMAGE_MENU_ITEM,
-					       "PanelPlaceMenuItem", &info, 0);
-	}
-
-	return type;
-}
-
-GType
-panel_desktop_menu_item_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static const GTypeInfo info = {
-			sizeof (PanelDesktopMenuItemClass),
-			NULL,
-			NULL,
-			(GClassInitFunc) panel_desktop_menu_item_class_init,
-			NULL,
-			NULL,
-			sizeof (PanelDesktopMenuItem),
-			0,
-			(GInstanceInitFunc) panel_desktop_menu_item_instance_init,
-			NULL
-		};
-
-		type = g_type_register_static (GTK_TYPE_IMAGE_MENU_ITEM,
-					       "PanelDesktopMenuItem", &info, 0);
-	}
-
-	return type;
-}
-
-
 GtkWidget *
 panel_place_menu_item_new (gboolean use_image)
 {
 	PanelPlaceMenuItem *menuitem;
-	GtkWidget          *accel_label;
+	GtkWidget          *image;
 
 	menuitem = g_object_new (PANEL_TYPE_PLACE_MENU_ITEM, NULL);
 
-	accel_label = gtk_accel_label_new (_("Places"));
-	gtk_misc_set_alignment (GTK_MISC (accel_label), 0.0, 0.5);
-
-	gtk_container_add (GTK_CONTAINER (menuitem), accel_label);
-	gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (accel_label),
-					  GTK_WIDGET (menuitem));
-	gtk_widget_show (accel_label);
-
-	menuitem->priv->use_image = use_image;
-	if (use_image) {
-		GtkWidget *image;
-
+	if (use_image)
 		image = gtk_image_new_from_icon_name ("gnome-fs-directory",
 						      panel_menu_icon_get_size ());
+	else
+		image = NULL;
 
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem),
-					       image);
-	}
+	setup_menuitem (GTK_WIDGET (menuitem),
+			image ? panel_menu_icon_get_size () : GTK_ICON_SIZE_INVALID,
+			image,
+			_("Places"),
+			TRUE);
+
+	menuitem->priv->use_image = use_image;
 
 	menuitem->priv->menu = panel_place_menu_item_create_menu (menuitem);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menuitem),
@@ -958,28 +923,23 @@ panel_desktop_menu_item_new (gboolean use_image,
 			     gboolean append_lock_logout)
 {
 	PanelDesktopMenuItem *menuitem;
-	GtkWidget            *accel_label;
+	GtkWidget            *image;
 
 	menuitem = g_object_new (PANEL_TYPE_DESKTOP_MENU_ITEM, NULL);
 
-	accel_label = gtk_accel_label_new (_("System"));
-	gtk_misc_set_alignment (GTK_MISC (accel_label), 0.0, 0.5);
-
-	gtk_container_add (GTK_CONTAINER (menuitem), accel_label);
-	gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (accel_label),
-					  GTK_WIDGET (menuitem));
-	gtk_widget_show (accel_label);
-
-	menuitem->priv->use_image = use_image;
-	if (use_image) {
-		GtkWidget *image;
-
+	if (use_image)
 		image = gtk_image_new_from_icon_name ("computer",
 						      panel_menu_icon_get_size ());
+	else
+		image = NULL;
 
-		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem),
-					       image);
-	}
+	setup_menuitem (GTK_WIDGET (menuitem),
+			image ? panel_menu_icon_get_size () : GTK_ICON_SIZE_INVALID,
+			image,
+			_("System"),
+			TRUE);
+
+	menuitem->priv->use_image = use_image;
 
 	menuitem->priv->append_lock_logout = append_lock_logout;
 	if (append_lock_logout)
